package de.uni_trier.de.webview;

import javafx.application.Application;
import javafx.beans.value.ObservableValue;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.ScrollEvent;
import javafx.scene.layout.*;
import javafx.scene.text.FontSmoothingType;
import javafx.stage.Stage;
import javafx.scene.web.*;

public class WebViewDemo extends Application {
    @Override
    public void start(Stage stage) {
        WebView webView = new WebView();
        WebEngine engine = webView.getEngine();
        webView.setZoom(0.80);
        webView.setFontSmoothingType(FontSmoothingType.GRAY);

        engine.titleProperty().addListener(
                (ObservableValue<? extends String> obs, String oldValue, String newValue) -> {
                    stage.setTitle(newValue);
                });

        webView.setOnScroll(event -> {
            //System.out.println(event.getDeltaY());
            if (event.getDeltaY() > 100)
                webView.setZoom(webView.getZoom() * 1.1);
            else
                webView.setZoom(webView.getZoom() * 0.9);
        });

        engine.load("https://www.google.de");

        double dist = 5.0;
        Insets insets = new Insets(dist, dist, dist, dist);

        TextField tfURL = new TextField();
        Button btnLoad = new Button("Laden");
        btnLoad.setDefaultButton(true);
        btnLoad.setOnAction(event -> {
            String s = tfURL.getText();
            if (!s.startsWith("https://") && !s.startsWith("http://"))
                s = "https://" + s;
            engine.load(s);
        });

        HBox top = new HBox();
        top.getChildren().addAll(tfURL, btnLoad);
        HBox.setHgrow(tfURL, Priority.ALWAYS);
        HBox.setMargin(btnLoad, new Insets(0, 0, 0, dist));

        BorderPane root = new BorderPane();
        root.setTop(top);
        root.setCenter(webView);
        BorderPane.setMargin(top, insets);
        BorderPane.setMargin(webView, insets);

        Scene scene = new Scene(root, 1024, 800);
        //stage.setTitle("JavaFX WebView Demo");
        stage.setScene(scene);
        stage.show();
    }
}